package tsp;

import java.util.Random;

public class Environment {

    private float distMatrix[][];		// distance matrix (problem instance)
    private long randSeed = 42;			// seed for random # generator
    private int nCities;			// number of cities for random problem instance
    private int initialPop = 10;		// initial population
    private int maxPop = 100;			// max population (for genocide)
    private int genPop = 50;			// genocided population (for genocide)
    private double timeBound = 5000;		// max time for search
    private boolean useBestSol = false;		// mutate the best solution or a randomly selected solution?
    private boolean useGenocide = false;	// use genocide?
    private Random randGen = null;		// random number generator

    public Environment(int nCities, float minDist, float maxDist) {
        // store number of cities
        this.nCities = nCities;

        // create empty distance matrix
        distMatrix = new float[nCities][nCities];

        // initialize random # generator
        randGen = new Random(randSeed);

        // populate distance matrix
        for (int i = 0; i < nCities; i++) {
            for (int j = i; j < nCities; j++) {
                if (i == j) {
                    // diagonal - a city is a distance of 0 from itself
                    distMatrix[i][j] = 0f;
                } else {
                    // fill both sides across diagonal with random value
                    distMatrix[i][j] = ((maxDist - minDist) * randGen.nextFloat()) + minDist;
                    distMatrix[j][i] = distMatrix[i][j];
                }
            }
        }
    }

    public Environment(int nCities, float minDist, float maxDist, int initialPop, int maxPop, int genPop, double timeBound, boolean useBestSol, boolean useGenocide) {
        // call "normal" constructor - populate distance matrix
        this(nCities, minDist, maxDist);

        // just store these environment variables
        this.initialPop = initialPop;
        this.maxPop = maxPop;
        this.genPop = genPop;
        this.timeBound = timeBound;
        this.useBestSol = useBestSol;
        this.useGenocide = useGenocide;
    }

    // A bunch of getter functions for environment variables
    public int getNCities() {
        return nCities;
    }

    public int getInitialPop() {
        return initialPop;
    }

    public int getMaxPop() {
        return maxPop;
    }

    public int getGenPop() {
        return genPop;
    }

    public double getTimeBound() {
        return timeBound;
    }

    public boolean getUseBestSol() {
        return useBestSol;
    }

    public boolean getUseGenocide() {
        return useGenocide;
    }

    // get the distance between two cities
    public float getDist(int c1, int c2) {
        return distMatrix[c1][c2];
    }

    // get the random # generator (need to use this consistently wherever random numbers are needed)
    public Random getRandGen() {
        return randGen;
    }

    // build a print-ready string representing the distance matrix
    public String toString() {
        String retStr = String.format("%3s", "");

        for (int i = 0; i < distMatrix.length; i++) {
            retStr += String.format("%5d", i);
        }

        retStr += "\n";

        for (int i = 0; i < distMatrix.length; i++) {
            retStr += String.format("%3d", i);
            for (int j = 0; j < distMatrix[i].length; j++) {
                retStr += String.format("%5.1f", distMatrix[i][j]);
            }
            retStr += "\n";
        }

        return retStr;
    }
}
