package tsp;

public class Main {

    public static void main(String[] args) {
        // construct a new Environment
        Environment env = new Environment(
                20,         // Number of cities 
                10.0f,      // Min city dist
                100.0f,     // Max city dist
                500,        // Initial Population Size (randomly generated)
                1000,       // Max Population Size (if using Genocide)
                100,        // Genocided Population Size (if using Genocide)
                5000,       // Time Resource Bound (ms)
                false,      // Use Best Solution (fitness bias)
                true        // Use Genocide
                );

        // output the randomly generated problem instance
        System.out.println(env.toString());

        // create a new state (which will initialize with randomly generated initial population)
        State state = new State(env);

        // print out current state (initial population)
        System.out.println(state.toString());

        // variable to keep track of best solution seen so far
        Solution bestSol = null;

        // start time and transition count
        long startTime = System.currentTimeMillis();
        int transCount = 0;

        // main search loop
        for (;; transCount++) {
            // perform transition
            state.Transition();

            // get the current best solution
            Solution newBest = state.getBestSol();

            // if this is a new current best, print it out
            if (!newBest.equals(bestSol)) {
                bestSol = newBest;
                System.out.println("New Best [" + transCount + "]: " + bestSol.toString() + " : " + bestSol.getFitness());
            }

            // if we've run over the time bound, break the loop
            if ((System.currentTimeMillis() - startTime) > env.getTimeBound()) {
                break;
            }
        }

        // print out total search time
        System.out.println("Total Time: " + (System.currentTimeMillis() - startTime) + "ms");
        // print out total # of transitions
        System.out.println("Total Transitions: " + transCount);
    }
}
